"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// var { List } = require("immutable");
var ChainWebSocket = require("./ChainWebSocket");
var GrapheneApi = require("./GrapheneApi");
var ChainConfig = require("./ChainConfig");

var ApisInstance = function () {
	function ApisInstance() {
		_classCallCheck(this, ApisInstance);
	}

	_createClass(ApisInstance, [{
		key: "connect",


		/** @arg {string} connection .. */
		value: function connect(cs) {
			var _this = this;

			var rpc_user = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "";
			var rpc_password = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "";


			// console.log("INFO\tApiInstances\tconnect\t", cs);

			if (typeof window !== "undefined" && window.location && window.location.protocol === "https:" && cs.indexOf("wss://") < 0) {
				throw new Error("Secure domains require wss connection");
			}

			this.ws_rpc = new ChainWebSocket(cs, this.statusCb);

			this.init_promise = this.ws_rpc.login(rpc_user, rpc_password).then(function () {
				// console.log("Login done");
				_this._db = new GrapheneApi(_this.ws_rpc, "database");
				_this._net = new GrapheneApi(_this.ws_rpc, "network_broadcast");
				_this._hist = new GrapheneApi(_this.ws_rpc, "history");
				_this._crypt = new GrapheneApi(_this.ws_rpc, "crypto");
				_this._secure = new GrapheneApi(_this.ws_rpc, "secure");
				var db_promise = _this._db.init().then(function () {
					//https://github.com/cryptonomex/graphene/wiki/chain-locked-tx
					return _this._db.exec("get_chain_id", []).then(function (_chain_id) {
						_this.chain_id = _chain_id;
						return ChainConfig.setChainId(_chain_id);
						//DEBUG console.log("chain_id1",this.chain_id)
					});
				});
				_this.ws_rpc.on_reconnect = function () {
					_this.ws_rpc.login(rpc_user, rpc_password).then(function () {
						_this._db.init().then(function () {
							if (_this.statusCb) _this.statusCb("reconnect");
						});
						_this._net.init();
						_this._hist.init();
						_this._crypt.init();
						_this._secure.init();
					});
				};
				return Promise.all([db_promise, _this._net.init(), _this._hist.init(), _this._crypt.init()
				// Temporary squash crypto API error until the API is upgraded everywhere
				.catch(function (e) {
					return console.error("ApiInstance\tCrypto API Error", e);
				}), _this._secure.init()]);
			});
		}
	}, {
		key: "close",
		value: function close() {
			if (this.ws_rpc) this.ws_rpc.close();
			this.ws_rpc = null;
		}
	}, {
		key: "db_api",
		value: function db_api() {
			return this._db;
		}
	}, {
		key: "network_api",
		value: function network_api() {
			return this._net;
		}
	}, {
		key: "history_api",
		value: function history_api() {
			return this._hist;
		}
	}, {
		key: "crypto_api",
		value: function crypto_api() {
			return this._crypt;
		}
	}, {
		key: "secure_api",
		value: function secure_api() {
			return this._secure;
		}
	}, {
		key: "setRpcConnectionStatusCallback",
		value: function setRpcConnectionStatusCallback(callback) {
			this.statusCb = callback;
		}
	}]);

	return ApisInstance;
}();

module.exports = ApisInstance;